# Stubs for pyspark.sql.context (Python 3.5)
#

from typing import Any, Callable, List, Optional, Tuple, TypeVar, Union

from pyspark.context import SparkContext
from pyspark.rdd import RDD
from pyspark.sql.dataframe import DataFrame
from pyspark.sql.session import SparkSession
from pyspark.sql.types import DataType, StructType, Row
from pyspark.sql.readwriter import DataFrameReader
from pyspark.sql.streaming import DataStreamReader, StreamingQueryManager 

from pandas.core.frame import DataFrame as PandasDataFrame  # type: ignore
from py4j.java_gateway import JavaObject  # type: ignore


T = TypeVar('T')

class SQLContext:
    sparkSession = ...  # type: Any
    def __init__(self, sparkContext, sparkSession: Optional[Any] = ..., jsqlContext: Optional[Any] = ...) -> None: ...
    @classmethod
    def getOrCreate(cls: type, sc: SparkContext) -> 'SQLContext':...
    def newSession(self) -> 'SQLContext': ...
    def setConf(self, key: str, value) -> None: ...
    def getConf(self, key: str, defaultValue: Optional[str] = ...) -> str: ...
    @property
    def udf(self) -> UDFRegistration: ...
    def range(self, start: int, end: Optional[int] = ..., step: int = ..., numPartitions: Optional[int] = ...) -> DataFrame: ...
    def registerFunction(self, name: str, f: Callable[..., Any], returnType: DataType = ...) -> None: ...
    def registerJavaFunction(self, name: str, javaClassName: str, returnType: Optional[DataType] = ...) -> None: ...
    def createDataFrame(self, data: Union[RDD[T], List[T], PandasDataFrame], schema: Optional[Union[List[str], Tuple[str, ...], DataType]] = ..., samplingRatio: Optional[float] = ..., verifySchema: bool = ...) -> DataFrame: ...
    def registerDataFrameAsTable(self, df: DataFrame, tableName: str) -> None: ...
    def dropTempTable(self, tableName: str) -> None: ...
    def createExternalTable(self, tableName: str, path: Optional[str] = ..., source: Optional[str] = ..., schema: Optional[StructType] = ..., **options) -> DataFrame: ...
    def sql(self, sqlQuery: str) -> DataFrame: ...
    def table(self, tableName: str) -> DataFrame: ...
    def tables(self, dbName: Optional[str] = ...) -> DataFrame: ...
    def tableNames(self, dbName: Optional[str] = ...) -> List[str]: ...
    def cacheTable(self, tableName: str) -> None: ...
    def uncacheTable(self, tableName: str) -> None: ...
    def clearCache(self) -> None: ...
    @property
    def read(self) -> DataFrameReader: ...
    @property
    def readStream(self) -> DataStreamReader: ...
    @property
    def streams(self) -> StreamingQueryManager: ...

class HiveContext(SQLContext):
    def __init__(self, sparkContext: SparkContext, jhiveContext: Optional[JavaObject] = ...) -> None: ...
    def refreshTable(self, tableName: str) -> None: ...

class UDFRegistration:
    sqlContext = ...  # SQLContext
    def __init__(self, sqlContext: SQLContext) -> None: ...
    def register(self, name: str, f: Callable[..., Any], returnType: DataType = ...) -> None: ...

